@@ -562,6 +562,8 @@ bool EngineStatusHandler::received(Message &msg)
 	msg.retValue() << ",waiting=" << locks;
     msg.retValue() << ",acceptcalls=" << lookup(Engine::accept(),Engine::getCallAcceptStates());
     msg.retValue() << ",congestion=" << Engine::getCongestion();
+    if (msg.getBoolValue("reset",false))
+	Engine::self()->resetMax();
     if (details) {
 	NamedIterator iter(Engine::runParams());
 	char sep = ';';
@@ -600,7 +602,7 @@ bool EngineEventHandler::received(Message &msg)
 	ev = new CapturedEvent(level,*text);
     else {
 	// build a full text with timestamp and sender
-	char tstamp[30];
+	char tstamp[32];
 	Debugger::Formatting fmt = Debugger::getFormatting();
 	if (Debugger::None == fmt)
 	    fmt = Debugger::Relative;
@@ -1182,6 +1184,8 @@ static int supervise(int initDelay)
 	    return err;
 	}
 	if (s_childpid == 0) {
+	    // Running in child now
+	    s_childpid = ::getpid();
 	    s_super_handle = wdogfd[1];
 	    ::close(wdogfd[0]);
 	    if (s_logrotator) {
@@ -1573,6 +1577,10 @@ int Engine::engineInit()
     s_params.addParam("runattempt",String(s_run_attempt));
 #ifndef _WINDOWS
     s_params.addParam("lastsignal",String(s_childsig));
+    if (s_childpid != -1)
+	s_params.addParam("childpid",String(s_childpid));
+    if (s_superpid != -1)
+	s_params.addParam("superpid",String(s_superpid));
 #endif
     s_params.addParam("minworkers",String(s_minworkers));
     s_params.addParam("maxworkers",String(s_maxworkers));
@@ -1584,7 +1592,7 @@ int Engine::engineInit()
     if (s_affinity)
 	s_params.addParam("affinity",s_affinity);
 
-    int nodeBits = s_cfg.getIntValue("general","nodebits",0,0,10);
+    int nodeBits = s_cfg.getIntValue("general","nodebits",0,0,12);
     if (nodeBits) {
 	int nodeId = s_cfg.getIntValue("general","nodeid",0);
 	if (0 <= nodeId && nodeId < (1 << nodeBits)) {
@@ -1652,6 +1660,8 @@ int Engine::engineInit()
 	    }
 	}
     }
+    // Reload configuration file so conditionals will take into account runtime parameters
+    s_cfg.load();
     vars = s_cfg.getSection("variables");
     if (vars) {
 	unsigned int n = vars->length();
@@ -3049,6 +3059,8 @@ int Engine::main(int argc, const char** argv, const char** env, RunMode mode, En
 #ifndef _WINDOWS
     if (supervised)
 	retcode = supervise(supervised);
+    else
+	s_childpid = ::getpid();
     if (retcode >= 0)
 	return retcode;
 #endif
