@@ -225,7 +225,7 @@ MessageHandler::MessageHandler(const char* name, unsigned priority,
 	const char* trackName, bool addPriority)
     : String(name),
       m_trackName(trackName), m_priority(priority),
-      m_unsafe(0), m_dispatcher(0), m_filter(0), m_counter(0)
+      m_unsafe(0), m_dispatcher(0), m_filter(0), m_filterRegexp(0), m_counter(0)
 {
     DDebug(DebugAll,"MessageHandler::MessageHandler('%s',%u,'%s',%s) [%p]",
 	name,priority,trackName,String::boolText(addPriority),this);
@@ -274,6 +274,7 @@ void MessageHandler::setFilter(NamedString* filter)
 {
     clearFilter();
     m_filter = filter;
+    m_filterRegexp = YOBJECT(Regexp,filter);
 }
 
 void MessageHandler::clearFilter()
@@ -281,6 +282,7 @@ void MessageHandler::clearFilter()
     if (m_filter) {
 	NamedString* tmp = m_filter;
 	m_filter = 0;
+	m_filterRegexp = 0;
 	delete tmp;
     }
 }
@@ -395,8 +397,14 @@ bool MessageDispatcher::dispatch(Message& msg)
     for (; l; l=l->next()) {
 	MessageHandler *h = static_cast<MessageHandler*>(l->get());
 	if (h && (h->null() || *h == msg)) {
-	    if (h->filter() && (*(h->filter()) != msg.getValue(h->filter()->name())))
-		continue;
+	    if (h->filter()) {
+		if (h->filterRegexp()) {
+		    if (!h->filterRegexp()->matches(msg.getValue(h->filter()->name())))
+			continue;
+		}
+		else if (*(h->filter()) != msg[h->filter()->name()])
+		    continue;
+	    }
 	    if (counting)
 		Thread::setCurrentObjCounter(h->objectsCounter());
 
