@@ -1222,7 +1222,7 @@ void XmlDomParser::endElement(const String& name)
     }
     if (m_current->getName() != name) {
 	setError(ReadingEndTag);
-	DDebug(this,DebugNote,
+	Debug(this,DebugNote,
 	    "Received end element for %s, but the expected one is for %s [%p]",
 	    name.c_str(),m_current->getName().c_str(),this);
 	return;
@@ -1310,46 +1310,7 @@ XmlFragment::XmlFragment()
 // Copy Constructor
 XmlFragment::XmlFragment(const XmlFragment& orig)
 {
-    ObjList* ob = orig.getChildren().skipNull();
-    for (;ob;ob = ob->skipNext()) {
-	XmlChild* obj = static_cast<XmlChild*>(ob->get());
-	if (obj->xmlElement()) {
-	    XmlElement* el = obj->xmlElement();
-	    if (el)
-		addChild(new XmlElement(*el));
-	    continue;
-	}
-	else if (obj->xmlCData()) {
-	    XmlCData* cdata = obj->xmlCData();
-	    if (cdata)
-		addChild(new XmlCData(*cdata));
-	    continue;
-	}
-	else if (obj->xmlText()) {
-	    const XmlText* text = obj->xmlText();
-	    if (text)
-		addChild(new XmlText(*text));
-	    continue;
-	}
-	else if (obj->xmlComment()) {
-	    XmlComment* comm = obj->xmlComment();
-	    if (comm)
-		addChild(new XmlComment(*comm));
-	    continue;
-	}
-	else if (obj->xmlDeclaration()) {
-	    XmlDeclaration* decl = obj->xmlDeclaration();
-	    if (decl)
-		addChild(new XmlDeclaration(*decl));
-	    continue;
-	}
-	else if (obj->xmlDoctype()) {
-	    XmlDoctype* doctype = obj->xmlDoctype();
-	    if (doctype)
-		addChild(new XmlDoctype(*doctype));
-	    continue;
-	}
-    }
+    copy(orig);
 }
 
 // Destructor
@@ -1399,6 +1360,30 @@ XmlChild* XmlFragment::removeChild(XmlChild* child, bool delObj)
     return ch;
 }
 
+// Copy other fragment into this one
+void XmlFragment::copy(const XmlFragment& other, XmlParent* parent)
+{
+    for (ObjList* o = other.getChildren().skipNull(); o; o = o->skipNext()) {
+	XmlChild* ch = static_cast<XmlChild*>(o->get());
+	if (ch->xmlElement())
+	    ch = new XmlElement(*(ch->xmlElement()));
+	else if (ch->xmlCData())
+	    ch = new XmlCData(*(ch->xmlCData()));
+	else if (ch->xmlText())
+	    ch = new XmlText(*(ch->xmlText()));
+	else if (ch->xmlComment())
+	    ch = new XmlComment(*(ch->xmlComment()));
+	else if (ch->xmlDeclaration())
+	    ch = new XmlDeclaration(*(ch->xmlDeclaration()));
+	else if (ch->xmlDoctype())
+	    ch = new XmlDoctype(*(ch->xmlDoctype()));
+	else
+	    continue;
+	ch->setParent(parent);
+	addChild(ch);
+    }
+}
+
 // Create a String from this XmlFragment
 void XmlFragment::toString(String& dump, bool escape, const String& indent,
     const String& origIndent, bool completeOnly, const String* auth,
@@ -1470,6 +1455,13 @@ XmlElement* XmlFragment::findElement(ObjList* list, const String* name, const St
     return e;
 }
 
+// Replaces all ${paramname} in fragment's children with the corresponding parameters
+void XmlFragment::replaceParams(const NamedList& params)
+{
+    for (ObjList* o = m_list.skipNull(); o; o = o->skipNext())
+	static_cast<XmlChild*>(o->get())->replaceParams(params);
+}
+
 
 /*
  * XmlDocument
@@ -1647,7 +1639,7 @@ XmlSaxParser::Error XmlDocument::loadFile(const char* file, int* error)
 
 // Save this xml document in a file
 int XmlDocument::saveFile(const char* file, bool esc, const String& indent,
-    bool completeOnly) const
+    bool completeOnly, const char* eoln) const
 {
     if (!file)
 	file = m_file;
@@ -1656,11 +1648,13 @@ int XmlDocument::saveFile(const char* file, bool esc, const String& indent,
     File f;
     int err = 0;
     if (f.openPath(file,true,false,true,false)) {
-	String eol("\r\n");
+	String eol(eoln);
+	if (eoln && !eol)
+	    eol = "\r\n";
 	write(f,esc,eol,indent,completeOnly);
 	err = f.error();
 	// Add an empty line
-	if (err >= 0)
+	if (err >= 0 && eol)
 	    f.writeData((void*)eol.c_str(),eol.length());
     }
     else
@@ -1678,6 +1672,15 @@ int XmlDocument::saveFile(const char* file, bool esc, const String& indent,
     return f.error();
 }
 
+// Replaces all ${paramname} in document's components with the corresponding parameters
+void XmlDocument::replaceParams(const NamedList& params)
+{
+    if (m_root)
+        m_root->replaceParams(params);
+    m_beforeRoot.replaceParams(params);
+    m_afterRoot.replaceParams(params);
+}
+
 
 /*
  * XmlChild
@@ -1703,13 +1706,13 @@ XmlElement::XmlElement(const NamedList& element, bool empty, XmlParent* parent)
 
 // Copy constructor
 XmlElement::XmlElement(const XmlElement& el)
-    : m_children(el.m_children),
-    m_element(el.getElement()), m_prefixed(0),
+    : m_element(el.getElement()), m_prefixed(0),
     m_parent(0), m_inheritedNs(0),
     m_empty(el.empty()), m_complete(el.completed())
 {
     setPrefixed();
     setInheritedNs(&el,true);
+    m_children.copy(el.m_children,this);
 }
 
 // Create an empty xml element
@@ -2009,6 +2012,15 @@ bool XmlElement::setXmlns(const String& name, bool addAttr, const String& value)
     return true;
 }
 
+// Replaces all ${paramname} in element's attributes and children with the
+//  corresponding parameters
+void XmlElement::replaceParams(const NamedList& params)
+{
+    m_children.replaceParams(params);
+    for (ObjList* o = m_element.paramList()->skipNull(); o; o = o->skipNext())
+	params.replaceParams(*static_cast<String*>(o->get()));
+}
+
 // Build an XML element from a list parameter
 XmlElement* XmlElement::param2xml(NamedString* param, const String& tag, bool copyXml)
 {
@@ -2218,6 +2230,13 @@ bool XmlText::onlySpaces()
     return true;
 }
 
+// Replaces all ${paramname} in text with the corresponding parameters
+void XmlText::replaceParams(const NamedList& params)
+{
+    params.replaceParams(m_text);
+}
+
+
 /*
  * XmlDoctype
  */
