@@ -37,6 +37,7 @@ public:
     virtual void initConstructor(JsFunction* construct)
 	{
 	    construct->params().addParam(new ExpFunction("keys"));
+	    construct->params().addParam(new ExpFunction("global"));
 	}
 protected:
     bool runNative(ObjList& stack, const ExpOperation& oper, GenObject* context);
@@ -529,7 +530,7 @@ void JsObject::internalToJSON(const GenObject* obj, bool isStr, String& buf, int
 	buf << String::boolText(oper->valBoolean());
     else if (oper->isNumber()) {
 	if (oper->isInteger())
-	    buf << oper->number();
+	    buf << *oper;
 	else
 	    buf << "null";
     }
@@ -682,8 +683,19 @@ bool JsObject::runAssign(ObjList& stack, const ExpOperation& oper, GenObject* co
 	ExpWrapper* w = YOBJECT(ExpWrapper,&oper);
 	if (w) {
 	    JsFunction* jsf = YOBJECT(JsFunction,w->object());
-	    if (jsf)
-		jsf->firstName(oper.name());
+	    if (jsf) {
+		ScriptRun* runner = YOBJECT(ScriptRun,context);
+		JsFunction* clonedFunc = jsf->cloneFunction(oper,runner && runner->context() ? runner->context()->mutex() : 0);
+		if (clonedFunc) {
+		    clonedFunc->firstName(oper.name());
+		    w = new ExpWrapper(clonedFunc,oper.name(),oper.barrier());
+		    w->lineNumber(oper.lineNumber());
+		    params().setParam(w);
+		    return true;
+		}
+		else
+		    jsf->firstName(oper.name());
+	    }
 	    params().setParam(w->clone(oper.name()));
 	}
 	else
@@ -879,8 +891,18 @@ bool JsObjectObj::runNative(ObjList& stack, const ExpOperation& oper, GenObject*
 	}
 	else
 	    return false;
-	const NamedList* lst = YOBJECT(NamedList,obj);
-	if (lst) {
+	ScriptContext* scr = YOBJECT(ScriptContext,obj);
+	if (scr) {
+	    ObjList names;
+	    scr->fillFieldNames(names);
+	    JsArray* jsa = new JsArray(context,oper.lineNumber(),mutex());
+	    for (ObjList* o = names.skipNull(); o; o = o->skipNext()) {
+		String* name = static_cast<String*>(o->get());
+		jsa->push(new ExpOperation(*name,0,true));
+	    }
+	    ExpEvaluator::pushOne(stack,new ExpWrapper(jsa,"keys"));
+	}
+	else if (const NamedList* lst = YOBJECT(NamedList,obj)) {
 	    NamedIterator iter(*lst);
 	    JsArray* jsa = new JsArray(context,oper.lineNumber(),mutex());
 	    while (const NamedString* ns = iter.get())
@@ -892,6 +914,16 @@ bool JsObjectObj::runNative(ObjList& stack, const ExpOperation& oper, GenObject*
 	    ExpEvaluator::pushOne(stack,JsParser::nullClone());
 	TelEngine::destruct(op);
     }
+    else if (oper.name() == YSTRING("global")) {
+	if (oper.number() != 0)
+	    return false;
+	ScriptRun* run = YOBJECT(ScriptRun,context);
+	JsObject* obj = YOBJECT(JsObject,run ? run->context() : context);
+	if (obj && obj->ref())
+	    ExpEvaluator::pushOne(stack,new ExpWrapper(obj,"global"));
+	else
+	    ExpEvaluator::pushOne(stack,JsParser::nullClone());
+    }
     else
 	return JsObject::runNative(stack,oper,context);
     return true;
@@ -1062,6 +1094,8 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 	// copy this array - only numerically indexed elements!
 	for (int i = 0; i < m_length; i++) {
 	    NamedString* ns = params().getParam(String(i));
+	    if (!ns)
+		continue;
 	    ExpOperation* op = YOBJECT(ExpOperation,ns);
 	    op = op ? op->clone() : new ExpOperation(*ns,ns->name(),true);
 	    array->params().addParam(op);
@@ -1074,10 +1108,14 @@ bool JsArray::runNative(ObjList& stack, const ExpOperation& oper, GenObject* con
 		int len = ja->length();
 		for (int i = 0; i < len; i++) {
 		    NamedString* ns = ja->params().getParam(String(i));
-		    ExpOperation* arg = YOBJECT(ExpOperation,ns);
-		    arg = arg ? arg->clone() : new ExpOperation(*ns,0,true);
-		    const_cast<String&>(arg->name()) = (unsigned int)array->m_length++;
-		    array->params().addParam(arg);
+		    if (ns) {
+			ExpOperation* arg = YOBJECT(ExpOperation,ns);
+			arg = arg ? arg->clone() : new ExpOperation(*ns,0,true);
+			const_cast<String&>(arg->name()) = (unsigned int)array->m_length++;
+			array->params().addParam(arg);
+		    }
+		    else
+			array->m_length++;
 		}
 		TelEngine::destruct(op);
 	    }
@@ -1620,22 +1658,20 @@ bool JsMath::runNative(ObjList& stack, const ExpOperation& oper, GenObject* cont
     XDebug(DebugAll,"JsMath::runNative() '%s' in '%s' [%p]",
 	oper.name().c_str(),toString().c_str(),this);
     if (oper.name() == YSTRING("abs")) {
-	if (!oper.number())
-	    return false;
-	int64_t n = 0;
+	int64_t n = ExpOperation::nonInteger();
 	for (int i = (int)oper.number(); i; i--) {
 	    ExpOperation* op = popValue(stack,context);
 	    if (op->isInteger())
 		n = op->number();
+	    else if (JsParser::isEmpty(*op))
+		n = 0;
 	    TelEngine::destruct(op);
 	}
-	if (n < 0)
+	if ((ExpOperation::nonInteger() != n) && (n < 0))
 	    n = -n;
 	ExpEvaluator::pushOne(stack,new ExpOperation(n));
     }
     else if (oper.name() == YSTRING("max")) {
-	if (!oper.number())
-	    return false;
 	int64_t n = LLONG_MIN;
 	for (int i = (int)oper.number(); i; i--) {
 	    ExpOperation* op = popValue(stack,context);
@@ -1643,11 +1679,11 @@ bool JsMath::runNative(ObjList& stack, const ExpOperation& oper, GenObject* cont
 		n = op->number();
 	    TelEngine::destruct(op);
 	}
+	if (LLONG_MIN == n)
+	    n = ExpOperation::nonInteger();
 	ExpEvaluator::pushOne(stack,new ExpOperation(n));
     }
     else if (oper.name() == YSTRING("min")) {
-	if (!oper.number())
-	    return false;
 	int64_t n = LLONG_MAX;
 	for (int i = (int)oper.number(); i; i--) {
 	    ExpOperation* op = popValue(stack,context);
@@ -1655,6 +1691,8 @@ bool JsMath::runNative(ObjList& stack, const ExpOperation& oper, GenObject* cont
 		n = op->number();
 	    TelEngine::destruct(op);
 	}
+	if (LLONG_MAX == n)
+	    n = ExpOperation::nonInteger();
 	ExpEvaluator::pushOne(stack,new ExpOperation(n));
     }
     else if (oper.name() == YSTRING("random")) {
