@@ -5,7 +5,7 @@
  * Wave file driver (record+playback)
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2014 Null Team
+ * Copyright (C) 2004-2020 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -123,6 +123,8 @@ public:
     virtual bool msgExecute(Message& msg, String& dest);
 protected:
     void statusParams(String& str);
+    bool canStopCall() const
+	{ return true; }
 private:
     AttachHandler* m_handler;
 };
@@ -166,6 +168,8 @@ typedef struct {
     uint32_t chan;
 } AuHeader;
 
+const char AuInfo[8] = "By YATE";
+
 #define ILBC_HEADER_LEN 9
 
 static const char* ilbcFormat(Stream& stream)
@@ -245,6 +249,11 @@ void WaveSource::init(const String& file, bool autorepeat)
 	m_format = "ilbc30";
     else if (file.endsWith(".g729"))
 	m_format = "g729";
+    else if (file.endsWith(".g722")) {
+	m_rate = 16000;
+	m_brate = 8000;
+	m_format = "g722/16000";
+    }
     else if (file.endsWith(".au"))
 	detectAuFormat();
     else if (file.endsWith(".wav"))
@@ -314,6 +323,10 @@ void WaveSource::detectAuFormat()
 	case 27:
 	    m_format = "alaw";
 	    break;
+	case 24:
+	    m_format = "g722";
+	    m_brate /= 2;
+	    break;
 	case 3:
 	    m_brate *= 2;
 	    m_swap = true;
@@ -556,6 +569,8 @@ WaveConsumer::WaveConsumer(const String& file, CallEndpoint* chan, unsigned maxl
 	m_format = "ilbc30";
     else if (file.endsWith(".g729"))
 	m_format = "g729";
+    else if (file.endsWith(".g722"))
+	m_format = "g722/16000";
     else if (file.endsWith(".lbc")) {
 	m_header = Ilbc;
 	if (!m_format.startsWith("ilbc"))
@@ -610,6 +625,13 @@ WaveConsumer::~WaveConsumer()
 	    Debug(&__plugin,DebugInfo,"WaveConsumer rate=" FMT64U " b/s",m_time);
 	}
     }
+    if (m_stream && (Au == m_header)) {
+	int64_t len = m_stream->length();
+	if ((len >= (int64_t)(sizeof(AuHeader) + sizeof(AuInfo))) && (m_stream->seek(8) == 8)) {
+	    uint32_t bytes = htonl(len - sizeof(AuHeader) - sizeof(AuInfo));
+	    m_stream->writeData(&bytes,sizeof(bytes));
+	}
+    }
     delete m_stream;
     m_stream = 0;
     s_statsMutex.lock();
@@ -649,16 +671,19 @@ void WaveConsumer::writeAuHeader()
 	header.form = htonl(1);
     else if (fmt == "alaw")
 	header.form = htonl(27);
+    else if (fmt == "g722")
+	header.form = htonl(24);
     else {
 	Debug(DebugMild,"Invalid au format '%s', not writing header",m_format.c_str());
 	return;
     }
     header.sign = htonl(0x2E736E64);
-    header.offs = htonl(sizeof(header));
+    header.offs = htonl(sizeof(header) + sizeof(AuInfo));
     header.freq = htonl(rate);
     header.chan = htonl(chans);
-    header.len = 0;
+    header.len = 0xFFFFFFFF;
     m_stream->writeData(&header,sizeof(header));
+    m_stream->writeData(AuInfo,sizeof(AuInfo));
 }
 
 bool WaveConsumer::setFormat(const DataFormat& format)
@@ -1130,6 +1155,10 @@ bool WaveFileDriver::msgExecute(Message& msg, String& dest)
     unsigned int maxlen = msg.getIntValue("maxlen");
     CallEndpoint* ch = YOBJECT(CallEndpoint,msg.userData());
     if (ch) {
+	if (msg.getBoolValue(YSTRING("stop_call"),false)) {
+	    msg.setParam(YSTRING("error"),"stopped_call");
+	    return false;
+	}
 	Debug(this,DebugInfo,"%s wave file '%s'", (meth ? "Record to" : "Play from"),
 	    dest.matchString(2).c_str());
 	WaveChan *c = new WaveChan(dest.matchString(2),meth,maxlen,msg,msg.getParam("callto"));
@@ -1159,6 +1188,7 @@ bool WaveFileDriver::msgExecute(Message& msg, String& dest)
     m.copyParam(msg,YSTRING("called"));
     m.copyParam(msg,YSTRING("caller"));
     m.copyParam(msg,YSTRING("callername"));
+    m.copyParam(msg,YSTRING("stop_call"));
     String callto(msg.getValue(YSTRING("direct")));
     if (callto.null()) {
 	const char *targ = msg.getValue(YSTRING("target"));
