@@ -336,7 +336,7 @@ while ($state != "") {
 		    $partycallid = $ev->GetValue("id");
 		    $ev->params["targetid"] = $ourcallid;
 		    $routeOnly = !Yate::Str2bool($ev->getValue("accept_call"));
-		    $interdigit = 1 * $ev->GetValue("interdigit",$interdigit);
+		    $interdigit = (int) $ev->GetValue("interdigit",$interdigit);
 		    $autoanswer = false;
 		    $callednum = "";
 		    $lang = $ev->getValue("lang");
